<?php
if(! defined('ABSPATH')){
	exit;
}
class BraintreeTransactionLog {
	
	private $fileName;
	private $filePath;
	private $file;
	
	public function __construct(){
		$this->fileName = 'transaction-log.txt';
		$this->filePath = BRAINTREE_PAYMENTS_LOC.'admin/log/';
	}
	
	public function writeToLogFile($object){
		$file = $this->openFile();
		if($file){
			if(is_a($object, Braintree_Exception_Authorization)){
				fwrite($file, $this->getTime().' - Configuration is incorrect. Check your API settings and your Merchant Account ID'."\n");
			}
			
			else if(is_a($object, Exception)){
				fwrite($file, $this->getTime().' '.$e->getMessage()."\n");
			}
			
			else if(is_string($object)){
				fwrite($file, $this->getTime().' '.$object."\n");
			}
			$this->closeFile($file);
		}
	}
	
	private function openFile(){
		if(file_exists($this->filePath.$this->fileName)){
			$file = fopen($this->filePath.$this->fileName, 'a');
		}
		else {
			$file = fopen($this->filePath.$this->fileName, 'w');
		}
		return $file;
	}
	
	private function readFile(){
		if(file_exists($this->filePath.$this->fileName)){
			$file = fopen($this->filePath.$this->fileName, 'r');
		}
		return $file;
	}
	
	private function closeFile($file){
		fclose($file);
	}
	private function getTime(){
		return date('m/d/Y:G:i:s', time());
	}
	
	public function readLogFile(){
		$file = $this->readFile();
		if($file){
			while(! feof($file)) {
				$line = fgets($file, 4096);
				if($line == false){
					break;
				}
				echo '<div class="transactionlog-line">'.$line.'</div>';
			}
			$this->closeFile($file);
		}
	}
	
	public function writeTransactionToLog(Braintree_Transaction $transaction, $type){
		$file = $this->openFile();
		if($file){
			if($transaction->paymentInstrumentType === 'credit_card'){
				$paymentMethod = $transaction->creditCardDetails->maskedNumber;
			}
			else if ($transaction->paymentInstrumentType === 'paypal_account'){
				$paymentMethod = 'Paypal - '.$transaction->paypalDetails->payerEmail;
			}
			$string = $this->getTime().', Transaction Type: '.$type.', Order ID: '.$transaction->orderId.
			', Amount: '.$transaction->amount.' '.$transaction->currencyIsoCode.', Transaction ID: '.$transaction->id.', Payment Method: '.$paymentMethod.', Customer Details: '.$transaction->customerDetails->firstName.' '.$transaction->customerDetails->lastName;
			$string = str_replace(["\r\n", "\n"], '', $string);
			fwrite($file, $string."\n");
		}
		$this->closeFile($file);
	}
	
	public function writeFailedTransactionToLog($result, $type){
		$file = $this->openFile();
		if($file){
			if($result->transaction){
				$transaction = $result->transaction;
				if($transaction->paymentInstrumentType === 'credit_card'){
					$paymentMethod = $result->transaction->creditCardDetails->maskedNumber;
				}
				else if ($transaction->paymentInstrumentType === 'paypal_account'){
					$paymentMethod = 'Paypal - '.$result->transaction->paypalDetails->payerEmail;
				}
				$string = $this->getTime().', Failure Message: '.$result->message.', Transaction Type: '.$type. 
				', Amount: '.$transaction->amount.' '.$transaction->currencyIsoCode.', Payment Method: '.$paymentMethod.', Customer Details: '.$transaction->customerDetails->firstName.' '.$transaction->customerDetails->lastName;;
				$string = str_replace(["\r\n", "\n"], '', $string);
				fwrite($file, $string."\n");
			}
			else {
				fwrite($result->message);
			}
		}
		$this->closeFile($file);
	}
}
?>