<?php
class Braintree_Donations {
	/**
	 * Contruct that configures the Braintree instance
	 */
	
	private $environment;
	
	private $merchantId;
	
	private $public_key;
	
	private $private_key;
	
	private $config;
	
	private $merchantAccountId;
	
	private $donation_form;
	
	private $paypal_enabled;
	
	public $require_firstName;
	
	public $require_lastName;
	
	public $require_postalCode;
	
	public $require_billingAddress;
	
	public function __construct(){
		$this->load_configuration();
	}
	public function get_config(){
		return $this->config;
	}
	private function load_configuration(){
		
		if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
			$_SERVER['HTTPS'] = 'on';
		
		$license_status = get_braintree_payments_config('braintree_payments_license_status');
		$this->config['api_keys'] = maybe_unserialize(get_braintree_payments_config('braintree_api_keys_config'));
		$this->config['messages'] = maybe_unserialize(get_braintree_payments_config('braintree_messages_config'));
		$this->config['merchant_accounts'] = maybe_unserialize(get_braintree_payments_config('braintree_merchant_account_id_config'));
		$this->config['donation_form_settings'] = get_braintree_payments_config('braintree_donation_form_settings');
		
		$production = ($this->config['api_keys']['environment'] === 'production') ? true : false;
		
		if($license_status == 'inactive'){
			check_braintree_donations_license();
			$license_status = get_braintree_payments_config('braintree_payments_license_status');
			$production = ($license_status == 'inactive') ? false : true;
		}
		
		if($production && !is_ssl()){
			$production = false;
		}
		
		$this->environment = ($production) ? 'production' : 'sandbox';
		$this->merchantId = ($production) ? $this->config['api_keys']['production']['merchantId'] : $this->config['api_keys']['sandbox']['merchantId'];
		$this->public_key = ($production) ? $this->config['api_keys']['production']['public_key'] : $this->config['api_keys']['sandbox']['public_key'];
		$this->private_key  = ($production) ? $this->config['api_keys']['production']['private_key'] : $this->config['api_keys']['sandbox']['private_key'];
		$this->merchantAccountId = ($production) 
		         ? $this->config['merchant_accounts']['production']['merchant_account_id'] :
		         $this->config['merchant_accounts']['sandbox']['merchant_account_id'];
		$this->donation_form = ($this->config['donation_form_settings']['donation_form'] == 'custom') ? 'custom' : 'dropin';
		$this->paypal_enabled = ($this->config['donation_form_settings']['paypal_active'] == 'true') ? true : false;
		$this->require_firstName = $this->config['donation_form_settings']['require_firstname'] == 'enabled' ? true : false;
		$this->require_lastName = $this->config['donation_form_settings']['require_lastname'] == 'enabled' ? true : false;
		$this->require_postalCode = $this->config['donation_form_settings']['require_postalcode'] == 'enabled' ? true : false;
		$this->require_billingAddress = $this->config['donation_form_settings']['require_billing_address'] == 'enabled' ? true : false;
		
		if(class_exists('Braintree_Configuration')){
		Braintree_Configuration::environment($this->environment);
		Braintree_Configuration::merchantId($this->merchantId);
		Braintree_Configuration::publicKey($this->public_key);
		Braintree_Configuration::privateKey($this->private_key);
		}
	}
	
	public function process_payment(){
		return $this->execute_payment();
	}
	
	private function execute_payment(){
		$log = new BraintreeTransactionLog();
		$amount = $_POST['donation_amount'];
		$billing_name = $_POST['billing_name'];
		//$nonce = ($this->environment == 'production') ? $_POST['payment_method_nonce'] : 'fake-valid-nonce';
		$nonce = $_POST['payment_method_nonce'];
		$payment_type = $_POST['payment_type'];
		try{
			if($this->require_firstName){
				if(empty($_POST['donation_firstname'])){
					throw new BraintreeDonationException('Please enter your first name.');
				}
			}
			if($this->require_lastName){
				if(empty($_POST['donation_lastname'])){
					throw new BraintreeDonationException('Please enter your last name.');
				}
			}
		}
		catch(Exception $e){
			return $e;
		}
		if($payment_type == 'CreditCard' && $this->require_billingAddress) {
			try {
				if(empty($_POST['billing_address'])){
				throw new BraintreeAddressException('Please enter your Billing Address and Postal Code');
				}
			}
			catch (BraintreeAddressException $e){
				return $e;
			}
		}
		if($payment_type == 'CreditCard' && $this->require_postalCode && $this->donation_form === 'custom') {
			try{
			if(empty($_POST['billing_postalcode'])) {
				throw new BraintreeAddressException('Please enter your Billing Address and Postal Code');
			}
			}
			catch(BraintreeAddressException $e){
				return $e;
			}
		}
		
		if($payment_type == 'CreditCard'){
			$cardType = $_POST['payment_type'];
			$cardId = $_POST['cardId'];
			
			$args = array('amount'=>$amount,
						  'merchantAccountId'=>$this->merchantAccountId,
						  'paymentMethodNonce'=>$nonce,
						  'options'=>array(
							'submitForSettlement'=>true
			), 'billing'=>array('streetAddress'=>$_POST['billing_address'], 
					'postalCode'=>$_POST['billing_postalcode'],
					'countryCodeAlpha2'=>$_POST['billing_country']
			), 'customer'=>array('firstName'=>$_POST['donation_firstname'], 
					'lastName'=>$_POST['donation_lastname']
			));
			
		}
		
		else if($_POST['payment_type'] == 'PayPalAccount'){
			
			$description = $this->config['messages']['receipt_message'];
			
			$args = array('amount'=>$amount,
						  'merchantAccountId'=>$this->merchantAccountId, 
						  'paymentMethodNonce'=>$nonce,
						  'options'=>array(
							'submitForSettlement'=>true, 'paypal'=>array('description'=>$description)
						  ), 'customer'=>array('firstName'=>$_POST['donation_firstname'], 
								'lastName'=>$_POST['donation_lastname']
			)
						  
			);
		}
		try{
			return  Braintree_Transaction::sale($args);
		}
		catch(Exception $e){
			$log->writeToLogFile($e);
			return $e;
		}
		
	}
	
	public function generate_button_html(){
		$hmtl  = '<div class="braintreeDontaation">
					<input type="submit" class="donation-button" id="dontation_button"/>
				</div>';
		
		return $html;
	}
	
	/**
	 * Method that returns the client token generated by using the merchant account keys
	 * @return Ambigous <multitype:, mixed>
	 */
	private function get_client_token($use_customer = true){
		if($use_customer && $user_id = wp_get_current_user()->ID){
			$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
			return Braintree_ClientToken::generate(array('customerId'=>$braintree_id));
		}
		try{
		return Braintree_ClientToken::generate();
		}
		catch(Exception $e){
			return false;
		}
	}
	
	public function get_donation_form(){
		
		global $post;
		$url = ($post) ? get_permalink($post->ID) : $_SERVER['PHP_SELF'];
		$active_environment = $this->get_active_environment();
		
		if($this->donation_form == 'custom'){
			if($this->paypal_enabled){
				$paypal = $this->get_paypal_container();
			}
			
		$html = '<div class="braintree-donations">
				<div class="close-donation"><img src="'.BRAINTREE_PAYMENTS_ASSETS.'images/close-donation.png"/></div>
				'.$active_environment.'
				<form id="braintree_donations_form" class="donation-form" method="post" action="'.$url.'">
					<input type="hidden" id="client_token" value="'.$this->get_client_token().'"/>		
				<div class="braintree-donation-radio"><input name="braintree-donation-paymentMethod" type="radio" value="creditCard"/><span>Credit Card Donation</span></div>		
				<div id="creditcard-donation-container" style="display: none">
					'.$this->get_requiredInputFields().'
					<div class="payment-info-container">
						<div class="payment-info"><span>'.__('Cardholder Name', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="cardholder_name" id="billing_name" value=""/></div>
					</div>
					<div class="payment-info-container">
						<div class="payment-info"><span>'.__('Credit Card Number', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="number" id="num_num" value=""/></div>
					</div>
				    <div class="payment-info-container-exp">
						<div class="payment-info"><span>'.__('Exp Date (MM/YYYY)', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="text" data-braintree-name="expiration_date" id="exp" value=""/></div>
					</div>
					<div class="payment-info-container-cvv">
						<div class="payment-info"><span>'.__('CVV', 'Braintree_Donations').'</span></div>
						<div class="payment-input"><input type="password" data-braintree-name="cvv" id="cvv" value=""/></div>
					</div>
					<div><button class="donation-form-button">'.__('Donate', 'Braintree_Donations') .'</button></div>
					</div><!--End creditcard-donation-container -->	
					  '.$paypal.'
					
					<div class="payment-info-container-donation">
						<div class="payment-info"><span>'.__('Donation Amount', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
						<div class="payment-input-donation"><div><span>'.$this->get_donation_currency(false).'</span></div><input type="text" id="donation_amount" value=""/></div>
					</div>
				    
				</form>
						</div>
				    		<div id="donation-message"></div>
		<div class="donations-overlay"></div>
		<ul class="loader">
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
   				<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
		</ul>
		<input type="hidden" id="ajax_url" value="'.admin_url().'admin-ajax.php' .'"/>';
		
		}
		else if($this->donation_form == 'dropin'){
			
			$html = '<div id="dropin-container" class="braintree-donations">
						<div class="close-donation"><img src="'.BRAINTREE_PAYMENTS_ASSETS.'images/close-donation.png"/></div>
						'.$active_environment.'
					<form id="braintree_donations_form" class="donation-form" method="post" action="'.$url.'">
							<input type="hidden" id="client_token" value="'.$this->get_client_token(true).'"/>
						'.$this->get_requiredInputFields().'
						<div class="payment-info-container">
							<div class="payment-info"><span>'.__('Donation Amount', 'Braintree_Donations').'</span></div>
							<div class="payment-input-donation"><span>'.$this->get_donation_currency().'</span><input type="text" id="donation_amount" value""/></div>
						</div>
						<div><button class="donation-form-button donation-form-button-dropin">'.__('Donate', 'Braintree_Donations') .'</button></div>
						</form>
					</div>
			<div id="donation-message"></div>
			<div class="donations-overlay"></div>
								<ul class="loader">
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
   				<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
  			<li>
    			<div class="circle"></div>
    			<div class="ball"></div>
  			</li>
		</ul>
			<input type="hidden" id="ajax_url" value="'.admin_url().'admin-ajax.php' .'"/>';
		}
		return $html;
	}
	
	public function get_donation_button(){
		$html = '<div class="braintree-donations-button">
						<button type="button" id="donate-button" 
				style="background-color: '.$this->config['donation_form_settings']['donation_button']['background_color'].'; border-color: '.$this->config['donation_form_settings']['donation_button']['border_color'].'">'
						.__('Donate', 'Braintree_Donations').'</button>
				 </div>';
		
		return $html;
			
	}
	
	private function get_donation_currency(){
		$current_currency = $this->config['api_keys'][$this->environment]['currency'];
		$currencies = Currencies::get_currencies();
		foreach($currencies as $currency => $symbol){
			if($current_currency == $currency){
				return $symbol;
			}
		}
	}
	
	public static function test_braintree_connection($args = ''){
		Braintree_Configuration::merchantId($args['merchantId']);
		Braintree_Configuration::environment($args['environment']);
		Braintree_Configuration::publicKey($args['publicKey']);
		Braintree_Configuration::privateKey($args['privateKey']);
		
		try{
		$response = Braintree_Customer::find('sags2352kj5h2jjgjk24bhjkf');
			if($response->success){
				$response['message'] = 'Connection to Braintree was successful';
				$response['code'] = 'success';
				return $response;
			}
		} 
		catch(Exception $e){
			if(is_a($e, 'Braintree_Exception_Authentication')){
				$response['message'] = 'Uh oh, your API keys are not correct. Please verify your entries';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_Authorization')){
				$response['message'] = 'Uh oh, you are not authorized to perform this function. Please contact Braintree';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_NotFound')){
				$response['message'] = 'Connection test to Braintree was successful. Let\'s make some dough!';
				$response['code'] = 'success';
			}
			if(is_a($e, 'Braintree_Exception_UpgradeRequired')){
				$response['message'] = 'Connection test to Braintree was successful. Let\'s make some dough!';
				$response['code'] = 'success';
			}
			if(is_a($e, 'Braintree_Exception_ServerError')){
				$response['message'] = 'Something has gone wrong on Braintree\'s server. Please test 
						again later';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_DownForMaintenance')){
				$response['message'] = 'Braintree\'s server\'s are down. Please try again later';
				$response['code'] = 'error';
			}
			if(is_a($e, 'Braintree_Exception_Unexpected')){
				$response['message'] = 'Houston, we have a problem. An unexptected error has occured. Please try the connection again';
				$response['code'] = 'error';
			}
			
			return $response;
		}
		
	}
	
	function get_paypal_container(){
		return '
				<div class="braintree-donation-radio"><input name="braintree-donation-paymentMethod" type="radio" value="paypal"/><span>Paypal Donation</span></div>			
				<div id="paypal-donation-container" style="display: none">
				<div class="payment-info-container-paypal">
				<div class="paypal-container">
				<div><span>Donate With Paypal</span></div>
					<div id="paypal-container"></div>
				<div class="cancel-paypal"><span id="cancel-paypal">Cancel Paypal</span></div>
				</div>
				</div>
				<input type="hidden" id="paypal_active"/>
				</div>';
	}
	private function get_active_environment(){
		if($this->environment == 'sandbox'){
			return '<div class="active-environment-message"><span>Sandbox Active</span></div>';
		}
	}
	
	private function get_requiredInputFields(){
		$html = '';
		if($this->require_firstName){
			$html .= '<div class="payment-info-container">
						<div class="payment-info"><span>'.__('First Name', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
						<div class="payment-input"><input type="text" name="donation_firstname" id="donation_firstname" value="" /></div>	
					</div>';
		}
		if($this->require_lastName){
			$html .= '<div class="payment-info-container">
						<div class="payment-info"><span>'.__('Last Name', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
						<div class="payment-input"><input type="text" name="donation_lastname" id="donation_lastname" value="" /></div>	
					</div>';
		}
		if($this->require_billingAddress) {
			$html .= '<div class="payment-info-container">
			<div class="payment-info"><span>'.__('Billing Address', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
			<div class="payment-input"><input type="text" name="billing_address" id="billing_address" value="" /></div>
			</div>'.$this->get_countryCodeAlpha2('billingAddress');
		}
		if($this->require_postalCode && $this->donation_form === 'custom'){
			$html .= '<div class="payment-info-container">
						<div class="payment-info-postal"><span>'.__('Postal Code', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
						<div class="payment-input-postal"><input type="text" name="billing_postcode" id="billing_postalcode" value="" /></div>
					  </div>'.$this->get_countryCodeAlpha2('postalCode');
		}
		if(! $this->require_billingAddress && ! $this->require_postalCode){
			$html .= $this->get_countryCodeAlpha2();
		}
		return $html;
	}
	
	public function get_countryCodeAlpha2($type){
		$html = '<div class="payment-info-container"><div class="payment-info-postal"><span>'.__('Billing Country', 'Braintree_Donations').'</span><span class="donation-required">*</span></div>
				<div class="payment-input-postal"><select name="billing_country" id="billing_country">';
		foreach(Braintree_Donation_Countries::$countries as $country => $text){
			$html .= '<option class="donation-billing-country" value="'. $country.'">'.$text.'</option>';
		}
		$html .= '</select></div></div>';
		if(('billingAddress' === $type && ! $this->require_postalCode) || $this->donation_form === 'dropin' ){
			return $html;
		}
		elseif ('postalCode' === $type && $this->require_postalCode && $this->donation_form === 'custom') {
			return $html;
		}
		elseif(! $this->require_billingAddress && ! $this->require_postalCode){
			return $html;
		}
	}
}
?>